@@ -1,5 +1,7 @@
 from argparse import ArgumentParser
 
+from ..utils import RED_TAG as RED
+from ..utils import RESET_TAG as RST
 from . import BaseAutoNLPCommand
 
 
@@ -10,8 +12,13 @@ def login_command_factory(args):
 class LoginCommand(BaseAutoNLPCommand):
     @staticmethod
     def register_subcommand(parser: ArgumentParser):
-        login_parser = parser.add_parser("login")
-        login_parser.add_argument("--api-key", type=str, required=True, help="API key")
+        login_parser = parser.add_parser("login", description="🔐 Logs you in AutoNLP!")
+        login_parser.add_argument(
+            "--api-key",
+            type=str,
+            required=True,
+            help="Your API key. You can find it under you user profile settings on huggingface.co!",
+        )
         login_parser.set_defaults(func=login_command_factory)
 
     def __init__(self, api_key: str):
@@ -22,3 +29,5 @@ class LoginCommand(BaseAutoNLPCommand):
 
         client = AutoNLP()
         client.login(token=self._api_key)
+
+        print(f"Welcome to 🤗 AutoNLP! Start by creating a project: {RED}autonlp create_project{RST}")
