@@ -1,51 +1,26 @@
 import sys
 from argparse import ArgumentParser, RawTextHelpFormatter
+from typing import Optional
 
 from loguru import logger
 
 from autonlp.validation import InvalidColMappingError, InvalidFileError
 
 from ..utils import BOLD_TAG as BLD
-from ..utils import CYAN_TAG as CYN
-from ..utils import GREEN_TAG as GRN
 from ..utils import RED_TAG as RED
 from ..utils import RESET_TAG as RST
 from . import BaseAutoNLPCommand
-
-
-COL_MAPPING_ARG_HELP = f"""\
-The files' column mapping. Must be like this:
-'{GRN}col_name{RST}:{CYN}autonlp_col_name{RST},{GRN}col_name{RST}:{CYN}autonlp_col_name{RST}'
-where '{CYN}autonlp_col_name{RST}' corresponds to an expected column in AutoNLP, and
-'{GRN}col_name{RST}' is the corresponding column in your files.
-"""
-
-COL_MAPPING_HELP = f"""\
-Expected columns for AutoNLP tasks:
---------------------------------------------------------
-
-{BLD}col_name1{RST} and {BLD}col_name2{RST} refer to columns in your files.
-
-{BLD}`binary_classification`{RST}:
-    {BLD}col_name1{RST} -> {BLD}text{RST}    (The text to classify)
-    {BLD}col_name2{RST} -> {BLD}target{RST}  (The label)
-    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}text{RST},{GRN}col_name2{RST}:{CYN}target{RST}'
-
-{BLD}`multi_class_classification`{RST}:
-    {BLD}col_name1{RST} -> {BLD}text{RST}    (The text to classify)
-    {BLD}col_name2{RST} -> {BLD}target{RST}  (The label)
-    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}text,{GRN}col_name2{RST}:{CYN}target{RST}'
-
-{BLD}`entity_extraction`{RST}:
-    {BLD}col_name1{RST} -> {BLD}tokens{RST}  (The tokens to tag)
-    {BLD}col_name2{RST} -> {BLD}tags{RST}    (The associated tags)
-    Example col_mapping: --col_mapping '{GRN}col_name1{RST}:{CYN}tokens{RST},{GRN}col_name2{RST}:{CYN}tags{RST}'
-
-"""
+from .common import COL_MAPPING_ARG_HELP, COL_MAPPING_HELP
 
 
 def upload_command_factory(args):
-    return UploadCommand(args.project, args.split, args.col_mapping, args.files)
+    return UploadCommand(
+        name=args.project,
+        split=args.split,
+        col_mapping=args.col_mapping,
+        files=args.files,
+        path_to_audio=args.path_to_audio,
+    )
 
 
 class UploadCommand(BaseAutoNLPCommand):
@@ -79,13 +54,24 @@ class UploadCommand(BaseAutoNLPCommand):
             required=True,
             help=COL_MAPPING_ARG_HELP,
         )
+        upload_parser.add_argument(
+            "--path_to_audio",
+            type=str,
+            default=None,
+            required=False,
+            help=(
+                "Required for speech recognition task. "
+                f"Comma-separated paths to {BLD}directories{RST} containing audio files"
+            ),
+        )
         upload_parser.set_defaults(func=upload_command_factory)
 
-    def __init__(self, name: str, split: str, col_mapping: str, files: str):
+    def __init__(self, name: str, split: str, col_mapping: str, files: str, path_to_audio: Optional[str]):
         self._name = name
         self._split = split
         self._col_mapping = col_mapping
         self._files = files
+        self._path_to_audio = path_to_audio
 
     def run(self):
         from ..autonlp import AutoNLP
@@ -106,7 +92,19 @@ class UploadCommand(BaseAutoNLPCommand):
 
         files = self._files.split(",")
         try:
-            project.upload(filepaths=files, split=self._split, col_mapping=col_maps)
+            project.upload(filepaths=files, split=self._split, col_mapping=col_maps, path_to_audio=self._path_to_audio)
+            print(
+                "🎉 Yupee! Your files have been uploaded.\n"
+                f"Once you're done, starting a training here: {RED}autonlp train --project {project.name}{RST}"
+            )
+        except ValueError as err:
+            logger.error("❌ Something went wrong!")
+            logger.error("Details:")
+            logger.error(str(err))
+        except FileNotFoundError as err:
+            logger.error("❌ One path you provided is invalid!")
+            logger.error("Details:")
+            logger.error(str(err))
         except InvalidFileError as err:
             logger.error("❌ Sorry, AutoNLP is not able to process the files you want to upload")
             logger.error("Details:")
@@ -117,8 +115,3 @@ class UploadCommand(BaseAutoNLPCommand):
             logger.error("Details:")
             for line in str(err).splitlines():
                 logger.error(line)
-
-        print(
-            "🎉 Yupee! Your files have been uploaded.\n"
-            f"Once you're done, starting a training here: {RED}autonlp train --project {project.name}{RST}"
-        )
